@@ -1,8 +1,8 @@
 
-__version__ = "0.7.4"
+__version__ = "0.8"
 __version_vcs_info__ = {
-    'branch': 'hotfix-0.7.4',
-    'revid': '430b262',
-    'revno': 593}
+    'branch': 'master',
+    'revid': 'ea04ad1',
+    'revno': 654}
 __version_user_email__ = "skalkoto@grnet.gr"
 __version_user_name__ = "Nikos Skalkotos"
